/*
	Emailscanner - The program filters email addresses from a file.
	Copyright 2020 Marek Küthe
	
	This program is free software. It comes without any warranty, to
	the extent permitted by applicable law. You can redistribute it
	and/or modify it under the terms of the Do What The Fuck You Want
	To Public License, Version 2, as published by Sam Hocevar. See
	http://www.wtfpl.net/ for more details.
*/

package emailscanner;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Finder {
	
	/**
	 * This is the extractor from which the lines are taken.
	 */
	protected Extractor ext;
	/**
	 * The LinkedHashSet type has the advantage that it cannot happen that an email address is entered there twice.
	 */
	protected LinkedHashSet<String> emails;
	
	/**
	 * The regular expression used to find the email address.
	 */
	public String RegExToFindEmail = "([A-Za-z0-9\\.]{3,}) {0,3}(?:(?:\\(|\\[)(?:at|aet)(?:\\)|\\])|@) {0,3}([A-Za-z0-9]+)+ {0,3}(?:(?:\\(|\\[)(?:punkt|dot)(?:\\)|\\])|\\.)? {0,3}([A-Za-z0-9]{2,10})";
	
	{
		ext = null;
		emails = new LinkedHashSet<String>();
	}
	
	public Finder(Extractor aext) {
		this.ext = aext;
	}
	
	/**
	 * Goes through each line that it receives from the extractor and checks for e-mail addresses using the regular expression. If he has found one, it is saved in the global LinkedHashSet emails.
	 */
	public void find() {
		//Pattern emailpat = Pattern.compile("([A-Za-z0-9\\.]{3,})\\s{0,3}(?:\\s{0,3}|\\(|\\[)?(?:" + at + ")(?:\\s{0,3}|\\)|\\])?\\s{0,3}([A-Za-z0-9]+)+\\s{0,3}(?:\\s{0,3}|\\(|\\[)?(?:" + dot + ")(?:\\s{0,3}|\\)|\\])?\\s{0,3}([A-Za-z0-9]{2,10})", Pattern.MULTILINE);

		Pattern emailpat = Pattern.compile(RegExToFindEmail);

		while(true) {
			String line = ext.getLine();
			if(line == null)
				break;
			
			Matcher searcher = emailpat.matcher(line);
			while(searcher.find()) {
				emails.add("" + searcher.group(1) + "@" + searcher.group(2) + "." + searcher.group(3));
				//System.out.println("" + searcher.group(1) + "@" + searcher.group(2) + "." + searcher.group(3));
			}
			
		}
	}
	
	/**
	 * Returns an iterator for the global LinkedHashSet.
	 * @return
	 */
	public Iterator<String> getEmailInterator() {
		return emails.iterator();
	}
	
}
